#include <avr/io.h>
#include <util/delay.h>		// Delay wird nur für Latch-Timig gebraucht -- bessere Möglichkeit ? Siehe   ---  lcd_latch()  ---

#include "config.h"
#include "opelLCD.h"
#include "spi.h"

#include "opelLCDSeg.h"		//defines der SegmentBitPos. Werden nur hier gebraucht.


volatile uint8_t	lcd_a,
			lcd_b,
			lcd_c,
			lcd_d,
			lcd_e,
			lcd_f,
			lcd_g,
			lcd_h,
			lcd_i,
			lcd_j,
			lcd_k,
			lcd_l;



void lcd_init ()
{
	lcd_a = lcd_b = lcd_c = lcd_d = lcd_e = lcd_f = lcd_g = lcd_h = lcd_i = lcd_j = lcd_k = lcd_l = 0;

	lcd_blank_ddr  |= (1<<lcd_blank_pin);		// Datenrichtungsregister für Blank und Latch setzen
	lcd_latch_ddr  |= (1<<lcd_latch_pin);

	lcd_blank_port |= (1<<lcd_blank_pin);		// Zuerst Blank, Schubsregister ausleeren, latchen, unblank

	spi_send(lcd_l);
	spi_send(lcd_k);
	spi_send(lcd_j);
	spi_send(lcd_i);
	spi_send(lcd_h);
	spi_send(lcd_g);
	spi_send(lcd_f);
	spi_send(lcd_e);
	spi_send(lcd_d);
	spi_send(lcd_c);
	spi_send(lcd_b);
	spi_send(lcd_a);

	lcd_latch_port |= (1<<lcd_latch_pin);

		lcd_blank_port &= ~(1<<lcd_blank_pin);		// Hoffe der PortZugriff ist lange genug für Latch Timing

	lcd_latch_port &= ~(1<<lcd_latch_pin);

	return;
}


void lcd_blank()	// nimmt die Ansteuerung weg.
{
	lcd_blank_port	|=	(1<<lcd_blank_pin);

	return;
}


void lcd_unblank()	// Steuert mit unveränderten Daten wieder an.
{
	lcd_blank_port	&=	~(1<<lcd_blank_pin);

	return;
}


void lcd_latch()	// Veränderte Daten aus Schieberegister in Latch zur Darstellung übernehmen.
{
	lcd_latch_port	|=	(1<<lcd_latch_pin);

		_delay_us (1);						// Diese Delay soll weg. unschön. Bääähhhh.

	lcd_latch_port	&=	~(1<<lcd_latch_pin);

	return;
}


void lcd_update()	// Schieberegister der LCD-Controller aus Hilfvariablen füllen. (12 Stück !!!)
{
	spi_send(lcd_l);
	spi_send(lcd_k);
	spi_send(lcd_j);
	spi_send(lcd_i);
	spi_send(lcd_h);
	spi_send(lcd_g);
	spi_send(lcd_f);
	spi_send(lcd_e);
	spi_send(lcd_d);
	spi_send(lcd_c);
	spi_send(lcd_b);
	spi_send(lcd_a);

	lcd_latch();

	return;
}


// Zum Spielen direktes setzen der Hilfsvariablen und direktes schreiben und latchen ins Display.
void lcd_do (uint8_t a,uint8_t b,uint8_t c,uint8_t d,uint8_t e,uint8_t f,uint8_t g,uint8_t h,uint8_t i,uint8_t j,uint8_t k,uint8_t l)
{
	lcd_a = a;
	lcd_b = b;
	lcd_c = c;
	lcd_d = d;
	lcd_e = e;
	lcd_f = f;
	lcd_g = g;
	lcd_h = h;
	lcd_i = i;
	lcd_j = j;
	lcd_k = k;
	lcd_l = l;

	lcd_update ();

	return;
}

// ClearScreen
void lcd_clear()
{
	spi_send(0);	// 1
	spi_send(0);	// 2
	spi_send(0);	// 3
	spi_send(0);	// 4
	spi_send(0);	// 5
	spi_send(0);	// 6
	spi_send(0);	// 7
	spi_send(0);	// 8
	spi_send(0);	// 9
	spi_send(0);	// 10
	spi_send(0);	// 11
	spi_send(0);	// 12

	lcd_latch();

	return;
}



//----------------------------------------------------------------------------------------------------------------
// Hier beginnen die "höheren" Funktionen.
//----------------------------------------------------------------------------------------------------------------


// Balkengrafik des linken Displays ansteuern.
void lcd_set_symbol_left (uint8_t w)
{
	if (w != 0)		lcd_f	|= 	 lcd_byte_f_symbol_left;
	else			lcd_f	&=	~lcd_byte_f_symbol_left;

	return;
}


// km/h Symbol im MittelDisplay ansteuern.
void lcd_set_symbol_middle (uint8_t w)
{
	if (w != 0)		lcd_l	|=	 lcd_byte_l_symbol_mid;
	else			lcd_l	&=	~lcd_byte_l_symbol_mid;

	return;
}


// Hintergrund des Drehzahlmessers ansteuern.
void lcd_set_symbol_right (uint8_t w)
{
	if (w != 0)		lcd_g	|=	 lcd_byte_g_symbol_right;
	else			lcd_g	&=	~lcd_byte_g_symbol_right;

	return;
}


// Tankanzeige "einstellen"
void lcd_set_tanklevel	(uint8_t tank, uint8_t w)
{
	// Die lever der Tankanzeige sind :
	// w -> Warnfeld !=0 an / 0==aus.
	// Level 1-12 niedrieg -> hoch


	// Zur Sicherheit zu grossen Wert abschneiden
	if (tank > 12)		tank 	= 	12;


	// bevor im switch die Felder aktiviert werden, erstmal vorläufig löschen.
				lcd_a	&=	~(
							lcd_byte_a_tank12
						);
				lcd_b	&= 	~(
							lcd_byte_b_tank4  |
							lcd_byte_b_tank5  |
							lcd_byte_b_tank6  |
							lcd_byte_b_tank7  |
							lcd_byte_b_tank8  |
							lcd_byte_b_tank9  |
							lcd_byte_b_tank10 |
							lcd_byte_b_tank11
						);
				lcd_c	&=	~(
							lcd_byte_c_tank1  |
							lcd_byte_c_tank2  |
							lcd_byte_c_tank3  |
							lcd_byte_c_tankwarn
						);


	// Hier werde alle Felder bis zum höchsten eingeschaltet, daher kein Break
	switch (tank)
	{
		case 0:		break;				// nix machen

		case 12:	lcd_a |= lcd_byte_a_tank12;
		case 11:	lcd_b |= lcd_byte_b_tank11;
		case 10:	lcd_b |= lcd_byte_b_tank10;
		case 9:		lcd_b |= lcd_byte_b_tank9 ;
		case 8:		lcd_b |= lcd_byte_b_tank8 ;
		case 7:		lcd_b |= lcd_byte_b_tank7 ;
		case 6:		lcd_b |= lcd_byte_b_tank6 ;
		case 5:		lcd_b |= lcd_byte_b_tank5 ;
		case 4:		lcd_b |= lcd_byte_b_tank4 ;
		case 3:		lcd_c |= lcd_byte_c_tank3 ;
		case 2:		lcd_c |= lcd_byte_c_tank2 ;
		case 1:		lcd_c |= lcd_byte_c_tank1 ;
	}


	// Warnfeld bearbeiten.
	if (w != 0)		lcd_c |= lcd_byte_c_tankwarn;

	return;
}


// Temperaturanzige "einstellen"
void lcd_set_templevel	(uint8_t temp, uint8_t w)
{
	// Die level der Tempanzeige sind :
	// w -> Warnfeld !=0 an / 0==aus.
	// Level 1-6 niedrieg -> hoch


	// Zur Sicherheit zu grossen Wert abschneiden.
	if (temp > 6)		temp 	=	 6;


	// bevor im switch die Felder aktiviert werden, erstmal vorläufig löschen.
				lcd_c	&=	~(
							lcd_byte_c_temp3 |
							lcd_byte_c_temp4 |
							lcd_byte_c_temp5 |
							lcd_byte_c_temp6
						);
				lcd_d	&= 	~(
							lcd_byte_d_temp1 |
							lcd_byte_d_temp2 |
							lcd_byte_d_tempwarn
						);


	// Hier werde alle Felder bis zum höchsten eingeschaltet.
	switch (temp)
	{
		case 0:		break;				// nix machen

		case 6:		lcd_c |= lcd_byte_c_temp6;
		case 5:		lcd_c |= lcd_byte_c_temp5;
		case 4:		lcd_c |= lcd_byte_c_temp4;
		case 3:		lcd_c |= lcd_byte_c_temp3;
		case 2:		lcd_d |= lcd_byte_d_temp2;
		case 1:		lcd_d |= lcd_byte_d_temp1;
	}


	// Warnfeld bearbeiten.
	if (w != 0)		lcd_d |= lcd_byte_d_tempwarn;

	return;
}


void lcd_set_voltlevel	(uint8_t volt, uint8_t w)
{
	// Die Volt sind :
	// w -> Warnfeld !=0 an / 0==aus.
	// Level 1-6 niedrieg -> hoch


	// Zur Sicherheit zu grossen Wert abschneiden.
	if (volt > 6)		volt 	= 	6;


	// bevor im switch die Felder aktiviert werden, erstmal vorläufig löschen.
				lcd_d	&=	~(
							lcd_byte_d_volt6 |
							lcd_byte_d_volt5 |
							lcd_byte_d_volt4 |
							lcd_byte_d_volt3 |
							lcd_byte_d_volt2
						);
				lcd_e	&=	~(
							lcd_byte_e_volt1 |
							lcd_byte_e_voltwarn
						);


	// Hier werde alle Felder bis zum höchsten eingeschaltet.
	switch (volt)
	{
		case 0:		break;				// nix machen	

		case 6:		lcd_d |= lcd_byte_d_volt6;
		case 5:		lcd_d |= lcd_byte_d_volt5;
		case 4:		lcd_d |= lcd_byte_d_volt4;
		case 3:		lcd_d |= lcd_byte_d_volt3;
		case 2:		lcd_d |= lcd_byte_d_volt2;
		case 1:		lcd_e |= lcd_byte_e_volt1;
	}


	// Warnfeld bearbeiten.
	if (w != 0) 		lcd_e |= lcd_byte_e_voltwarn;

	return;
}


void lcd_set_oillevel	(uint8_t oil , uint8_t w)
{
	// Die Öllevel sind :
	// w -> Warnfeld !=0 an / 0==aus.
	// Level 1-5 niedrieg -> hoch


	// Zur Sicherheit zu grossen Wert abschneiden.
	if (oil > 5)		oil 	= 	5;


	// bevor im switch die Felder aktiviert werden, erstmal vorläufig löschen.
				lcd_e	&=	~(
							lcd_byte_e_oil1 |
							lcd_byte_e_oil2 |
							lcd_byte_e_oil3 |
							lcd_byte_e_oil4 |
							lcd_byte_e_oil5 |
							lcd_byte_e_oilwarn
						);


	// Hier werde alle Felder bis zum höchsten eingeschaltet.
	switch (oil)
	{
		case 0:		break;				// nix machen	

		case 5:		lcd_e |= lcd_byte_e_oil5;
		case 4:		lcd_e |= lcd_byte_e_oil4;
		case 3:		lcd_e |= lcd_byte_e_oil3;
		case 2:		lcd_e |= lcd_byte_e_oil2;
		case 1:		lcd_e |= lcd_byte_e_oil1;
	}


	// Warnfeld bearbeiten.
	if (w != 0)		lcd_e |= lcd_byte_e_oilwarn;

	return;
}


void lcd_set_digit_1	(uint8_t n)
{

	if (n > 3)		n 	= 	3;


				lcd_f	&=	~(
							lcd_byte_f_s1_b |
							lcd_byte_f_s1_c |
							lcd_byte_f_s1_adeg
						);


	switch (n)
	{
		case 1:		lcd_f	|=	(
							lcd_byte_f_s1_b |
							lcd_byte_f_s1_c
						);
				break;

		case 2:		lcd_f	|=	(
							lcd_byte_f_s1_b |
							lcd_byte_f_s1_adeg
						);
		case 3:		break;
	}

	return;
}


void lcd_set_digit_2	(uint8_t n)
{
	if (n > 10)		n 	= 	10;


				lcd_f	&=	~(
							lcd_byte_f_s2_e |
							lcd_byte_f_s2_d |
							lcd_byte_f_s2_g
						);
				lcd_k	&=	~(
							lcd_byte_k_s2_a |
							lcd_byte_k_s2_b |
							lcd_byte_k_s2_c |
							lcd_byte_k_s2_f
						);


	switch (n)
	{
		case	0:	lcd_f	|=	(
							lcd_byte_f_s2_e |
							lcd_byte_f_s2_d
					 	);
				lcd_k	|=	(
							lcd_byte_k_s2_a |
							lcd_byte_k_s2_b |
							lcd_byte_k_s2_c |
							lcd_byte_k_s2_f
						);
				break;

		case	1:	lcd_k	|=	(
							lcd_byte_k_s2_b |
							lcd_byte_k_s2_c
						);
				break;

		case	2:	lcd_f	|=	(
							lcd_byte_f_s2_e |
							lcd_byte_f_s2_d |
							lcd_byte_f_s2_g
					 	);

				lcd_k	|=	(
							lcd_byte_k_s2_a |
							lcd_byte_k_s2_b
						);
				break;

		case	3:	lcd_f	|=	(
							lcd_byte_f_s2_d |
							lcd_byte_f_s2_g
					 	);

				lcd_k	|=	(
							lcd_byte_k_s2_a |
							lcd_byte_k_s2_b |
							lcd_byte_k_s2_c
						);
				break;

		case	4:	lcd_f	|=	(
							lcd_byte_f_s2_g
						);
				lcd_k	|=	(
							lcd_byte_k_s2_b |
							lcd_byte_k_s2_c |
							lcd_byte_k_s2_f
						);
				break;

		case	5:	lcd_f	|=	(
							lcd_byte_f_s2_d |
							lcd_byte_f_s2_g 
					 	);

				lcd_k	|=	(
							lcd_byte_k_s2_a |
							lcd_byte_k_s2_c |
							lcd_byte_k_s2_f
						);
				break;

		case	6:	lcd_f	|=	(
							lcd_byte_f_s2_e |
							lcd_byte_f_s2_d |
							lcd_byte_f_s2_g
					 	);

				lcd_k	|=	(
							lcd_byte_k_s2_a |
							lcd_byte_k_s2_c |
							lcd_byte_k_s2_f
						);
				break;

		case	7:	lcd_k	|=	(
							lcd_byte_k_s2_a |
							lcd_byte_k_s2_b |
							lcd_byte_k_s2_c
						);
				break;

		case	8:	lcd_f	|=	(
							lcd_byte_f_s2_e |
							lcd_byte_f_s2_d |
							lcd_byte_f_s2_g
						);
				lcd_k	|=	(
							lcd_byte_k_s2_a |
							lcd_byte_k_s2_b |
							lcd_byte_k_s2_c |
							lcd_byte_k_s2_f
					 	);
				break;

		case	9:	lcd_f	|=	(
							lcd_byte_f_s2_d |
							lcd_byte_f_s2_g
					 	);
				lcd_k	|=	(
							lcd_byte_k_s2_a |
							lcd_byte_k_s2_b |
							lcd_byte_k_s2_c |
							lcd_byte_k_s2_f
						);
				break;

		case	10:	break;
	}

	return;
}


void lcd_set_digit_3	(uint8_t n)
{

	if (n > 10)		n 	= 	10;


				lcd_l	&=	~(
							lcd_byte_l_s3_a |
							lcd_byte_l_s3_b |
							lcd_byte_l_s3_c |
							lcd_byte_l_s3_f |
							lcd_byte_l_s3_g
						);
				lcd_k	&=	~(
							lcd_byte_k_s3_e |
							lcd_byte_k_s3_d
						);


	switch (n)
	{
		case	0:	lcd_k	|=	(
							lcd_byte_k_s3_e |
							lcd_byte_k_s3_d
					 	);
				lcd_l	|=	(
							lcd_byte_l_s3_a |
							lcd_byte_l_s3_b |
							lcd_byte_l_s3_c |
							lcd_byte_l_s3_f
						);
				break;

		case	1:	lcd_l	|=	(
							lcd_byte_l_s3_b |
							lcd_byte_l_s3_c
						);
				break;

		case	2:	lcd_k	|=	(
							lcd_byte_k_s3_e |
							lcd_byte_k_s3_d
					 	);

				lcd_l	|=	(
							lcd_byte_l_s3_a |
							lcd_byte_l_s3_b |
							lcd_byte_l_s3_g
						);
				break;

		case	3:	lcd_k	|=	(
							lcd_byte_k_s3_d
					 	);

				lcd_l	|=	(
							lcd_byte_l_s3_a |
							lcd_byte_l_s3_b |
							lcd_byte_l_s3_c |
							lcd_byte_l_s3_g
						);
				break;

		case	4:
				lcd_l	|=	(
							lcd_byte_l_s3_b |
							lcd_byte_l_s3_c |
							lcd_byte_l_s3_f |
							lcd_byte_l_s3_g
						);
				break;

		case	5:	lcd_k	|=	(
							lcd_byte_k_s3_d
					 	);

				lcd_l	|=	(
							lcd_byte_l_s3_a |
							lcd_byte_l_s3_c |
							lcd_byte_l_s3_f |
							lcd_byte_l_s3_g
						);
				break;

		case	6:	lcd_k	|=	(
							lcd_byte_k_s3_e |
							lcd_byte_k_s3_d
					 	);

				lcd_l	|=	(
							lcd_byte_l_s3_a |
							lcd_byte_l_s3_c |
							lcd_byte_l_s3_f |
							lcd_byte_l_s3_g
						);
				break;

		case	7:
				lcd_l	|=	(
							lcd_byte_l_s3_a |
							lcd_byte_l_s3_b |
							lcd_byte_l_s3_c
						);
				break;

		case	8:	lcd_k	|=	(
							lcd_byte_k_s3_e |
							lcd_byte_k_s3_d
					 	);
				lcd_l	|=	(
							lcd_byte_l_s3_a |
							lcd_byte_l_s3_b |
							lcd_byte_l_s3_c |
							lcd_byte_l_s3_f |
							lcd_byte_l_s3_g
						);
				break;

		case	9:	lcd_k	|=	(
							lcd_byte_k_s3_d
					 	);

				lcd_l	|=	(
							lcd_byte_l_s3_a |
							lcd_byte_l_s3_b |
							lcd_byte_l_s3_c |
							lcd_byte_l_s3_f |
							lcd_byte_l_s3_g
						);
				break;
		case	10:	break;
	}

	return;
}


void lcd_set_upm	(uint8_t n)
{
	// Die Drehzallevel sind :


	// Zur Sicherheit zu grossen Wert abschneiden.
	if (n > 33) 	n = 33;


	// bevor im switch die Felder aktiviert werden, erstmal vorläufig löschen.
				lcd_k	&=	~(
							lcd_byte_k_dzm1 |
							lcd_byte_k_dzm2
						);
				lcd_j	&=	~(
							lcd_byte_j_dzm3 |
							lcd_byte_j_dzm4 |
							lcd_byte_j_dzm5 |
							lcd_byte_j_dzm6 |
							lcd_byte_j_dzm7 |
							lcd_byte_j_dzm8 |
							lcd_byte_j_dzm9 |
							lcd_byte_j_dzm10
						);
				lcd_i	&=	~(
							lcd_byte_i_dzm11 |
							lcd_byte_i_dzm12 |
							lcd_byte_i_dzm13 |
							lcd_byte_i_dzm14 |
							lcd_byte_i_dzm15 |
							lcd_byte_i_dzm16 |
							lcd_byte_i_dzm17 |
							lcd_byte_i_dzm18
						);
				lcd_h	&=	~(
							lcd_byte_h_dzm19 |
							lcd_byte_h_dzm20 |
							lcd_byte_h_dzm21 |
							lcd_byte_h_dzm22 |
							lcd_byte_h_dzm23 |
							lcd_byte_h_dzm27 |
							lcd_byte_h_dzm28 |
							lcd_byte_h_dzm29
						);
				lcd_g	&=	~(
							lcd_byte_g_dzm30 |
							lcd_byte_g_dzm31 |
							lcd_byte_g_dzm32 |
							lcd_byte_g_dzm33 |
							lcd_byte_g_dzm26 |
							lcd_byte_g_dzm25 |
							lcd_byte_g_dzm24 
						);


	// Hier werde alle Felder bis zum höchsten eingeschaltet.
	switch (n)
	{
		case 0:		break;				// nix machen
				
		case 33:	lcd_g |= lcd_byte_g_dzm33;
		case 32:	lcd_g |= lcd_byte_g_dzm32;
		case 31:	lcd_g |= lcd_byte_g_dzm31;
		case 30:	lcd_g |= lcd_byte_g_dzm30;
		case 29:	lcd_h |= lcd_byte_h_dzm29;
		case 28:	lcd_h |= lcd_byte_h_dzm28;
		case 27:	lcd_h |= lcd_byte_h_dzm27;
		case 26:	lcd_g |= lcd_byte_g_dzm26;
		case 25:	lcd_g |= lcd_byte_g_dzm25;
		case 24:	lcd_g |= lcd_byte_g_dzm24;
		case 23:	lcd_h |= lcd_byte_h_dzm23;
		case 22:	lcd_h |= lcd_byte_h_dzm22;
		case 21:	lcd_h |= lcd_byte_h_dzm21;
		case 20:	lcd_h |= lcd_byte_h_dzm20;
		case 19:	lcd_h |= lcd_byte_h_dzm19;
		case 18:	lcd_i |= lcd_byte_i_dzm18;
		case 17:	lcd_i |= lcd_byte_i_dzm17;
		case 16:	lcd_i |= lcd_byte_i_dzm16;
		case 15:	lcd_i |= lcd_byte_i_dzm15;
		case 14:	lcd_i |= lcd_byte_i_dzm14;
		case 13:	lcd_i |= lcd_byte_i_dzm13;
		case 12:	lcd_i |= lcd_byte_i_dzm12;
		case 11:	lcd_i |= lcd_byte_i_dzm11;
		case 10:	lcd_j |= lcd_byte_j_dzm10;
		case  9:	lcd_j |= lcd_byte_j_dzm9;
		case  8:	lcd_j |= lcd_byte_j_dzm8;
		case  7:	lcd_j |= lcd_byte_j_dzm7;
		case  6:	lcd_j |= lcd_byte_j_dzm6;
		case  5:	lcd_j |= lcd_byte_j_dzm5;
		case  4:	lcd_j |= lcd_byte_j_dzm4;
		case  3:	lcd_j |= lcd_byte_j_dzm3;
		case  2:	lcd_k |= lcd_byte_k_dzm2;
		case  1:	lcd_k |= lcd_byte_k_dzm1;
	}

	return;
}
